#!/usr/bin/env python3
###
### Script to check water conservation in the IPSL coupled model
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
## SVN information
#  $Author$
#  $Date$
#  $Revision$
#  $Id$
#  $HeadURL$

###
## Import system modules
import sys, os, shutil, subprocess, platform
import numpy as np
import configparser, re

## Creates parser
config = configparser.ConfigParser()
config.optionxform = str # To keep capitals

## Read experiment parameters
ATM=None ; ORCA=None ; NEMO=None ; OCE_relax=False ; OCE_icb=False ; Coupled=False ; Routing=None ;  TarRestartPeriod_beg=None ; TarRestartPeriod_end=None ; Comment=None ; Period=None ; Title=None

##
ARCHIVE =None ; STORAGE = None ; SCRATCHDIR=None ; R_IN=None ; rebuild=None
TmpDir=None ; FileOut=None ; dir_ATM_his=None ; dir_SRF_his=None ; dir_OCE_his=None ; dir_ICE_his=None ; FileCommon=None
file_ATM_his=None ; file_SRF_his=None ; file_RUN_his=None ; file_OCE_his=None ;  file_ICE_his=None ; file_OCE_sca=None
file_restart_beg=None ; file_restart_end=None ; file_ATM_beg=None ; file_ATM_end=None ; file_DYN_beg=None ; file_DYN_end=None ; file_SRF_beg=None ; file_SRF_end=None
file_RUN_beg=None ; file_RUN_end=None ; file_RUN_end=None ; file_OCE_beg=None ; file_ICE_beg=None ; file_OCE_beg=None ; file_OCE_end=None

d_ATM_his=None ; d_SRF_his=None ; d_RUN_his=None ; d_OCE_his=None ;  d_ICE_his=None ; d_OCE_sca=None
d_restart_beg=None ; d_restart_end=None ; d_ATM_beg=None ; d_ATM_end=None ; d_DYN_beg=None ; d_DYN_end=None ; d_SRF_beg=None ; d_SRF_end=None
d_RUN_beg=None ; d_RUN_end=None ; d_RUN_end=None ; d_OCE_beg=None ; d_ICE_beg=None ; d_OCE_beg=None ; d_OCE_end=None

# Arguments passed
print ( "Name of Python script:", sys.argv[0] )
IniFile = sys.argv[1]
print ("Input file : ", IniFile )
config.read (IniFile)
FullIniFile = 'full_' + IniFile

def setBool (chars) :
    '''Convert specific char string in boolean if possible'''
    setBool = chars
    for key in configparser.ConfigParser.BOOLEAN_STATES.keys () :
        if chars.lower() == key : setBool = configparser.ConfigParser.BOOLEAN_STATES[key]
    return setBool

def setNum (chars) :
    '''Convert specific char string in integer or real if possible'''
    if type (chars) == str :
        realnum = re.compile ("^[-+]?[0-9]*\.?[0-9]+(e[-+]?[0-9]+)?$")
        isReal = realnum.match(chars.strip()) != None
        isInt  = chars.strip().isdigit()
        if isReal :
            if isInt : setNum = int   (chars)
            else     : setNum = float (chars)
        else : setNum = chars
    else : setNum = chars
    return setNum

def setNone (chars) :
    '''Convert specific char string to None if possible'''
    if type (chars) == str :
        if chars in ['None', 'NONE', 'none'] : setNone = None
        else : setNone = chars
    else : setNone = chars
    return setNone

## Reading config
for Section in ['Experiment', 'libIGCM', 'Files', 'Physics' ] :
    if Section in config.keys() : 
        print ( f'[{Section}]' )
        for VarName in config[Section].keys() :
            locals()[VarName] = config[Section][VarName]
            locals()[VarName] = setBool (locals()[VarName])
            locals()[VarName] = setNum  (locals()[VarName])
            print ( '{:25} set to : {:}'.format (VarName, locals()[VarName]) )

if not 'Files' in config.keys() : config['Files'] = {}

def unDefined (char) :
    if char in globals () :
        if char == None : return True
        else : return False
    else : return True

##-- Some physical constants
#-- Earth Radius
if not 'Ra'            in locals () : Ra = 6366197.7236758135
#-- Gravity
if not 'Grav'          in locals () : Grav = 9.81
#-- Ice volumic mass (kg/m3) in LIM3
if not 'ICE_rho_ice'   in locals () : ICE_rho_ice = 917.0
#-- Snow volumic mass (kg/m3) in LIM3
if not 'ICE_rho_sno'   in locals () : ICE_rho_sno = 330.0
#-- Ocean water volumic mass (kg/m3) in NEMO
if not 'OCE_rho_liq'   in locals () : OCE_rho_liq = 1026.
#-- Water volumic mass in atmosphere
if not 'ATM_rho'       in locals () : ATM_rho = 1000.
#-- Water volumic mass in surface reservoirs
if not 'SRF_rho'       in locals () : SRF_rho = 1000.
#-- Water volumic mass of rivers
if not 'RUN_rho'       in locals () : RUN_rho = 1000.
#-- Year length
if not 'YearLength'    in locals () : YearLength = 365.25 * 24. * 60. * 60.

config['Physics'] = { 'Ra':Ra, 'Grav':Grav, 'ICE_rho_ice':ICE_rho_ice, 'ICE_rho_sno':ICE_rho_sno, 'OCE_rho_liq':OCE_rho_liq, 'ATM_rho':ATM_rho, 'SRF_rho':SRF_rho, 'RUN_rho':RUN_rho}

## --
ICO  = ( 'ICO' in ATM )
LMDZ = ( 'LMD' in ATM )
    

# Where do we run ?
SysName, NodeName, Release, Version, Machine = os.uname ()
TGCC  = ( 'irene'   in NodeName )
IDRIS = ( 'jeanzay' in NodeName )

## Set site specific libIGCM directories, and other specific stuff
if TGCC :
    CPU = subprocess.getoutput ( 'lscpu | grep "Model name"' )
    if "Intel(R) Xeon(R) Platinum" in CPU : Machine = 'irene'
    if "AMD"                       in CPU : Machine = 'irene-amd'

    if libIGCM : 
        if ARCHIVE    == None : ARCHIVE     = subprocess.getoutput ( f'ccc_home --cccstore   -d {Project} -u {User}' )
        if STORAGE    == None : STORAGE     = subprocess.getoutput ( f'ccc_home --cccwork    -d {Project} -u {User}' )
        if SCRATCHDIR == None : SCRATCHDIR  = subprocess.getoutput ( f'ccc_home --cccscratch -d {Project} -u {User}' )
        if R_IN       == None : R_IN        = os.path.join ( subprocess.getoutput ( f'ccc_home --cccwork -d igcmg -u igcmg' ), 'IGCM')
        if rebuild    == None : rebuild     = os.path.join ( subprocess.getoutput ( f'ccc_home --ccchome -d igcmg -u igcmg' ), 'Tools', Machine, 'rebuild_nemo', 'bin', 'rebuild_nemo' )

    ## Specific to run at TGCC.
    # Needed before importing a NetCDF library (netCDF4, xarray, cmds, etc ...)
    import mpi4py
    mpi4py.rc.initialize = False
        
    ## Creates output directory
    if TmpDir == None :
        TmpDir = os.path.join ( subprocess.getoutput ( 'ccc_home --cccscratch' ), f'WATER_{JobName}_{YearBegin}_{YearEnd}' )
        config['Files']['TmpDir'] = TmpDir
        
if IDRIS :
    raise Exception ("Pour IDRIS : repertoires et chemins a definir") 

config['System'] = {'SysName':SysName, 'NodeName':NodeName, 'Release':Release, 'Version':Version,'Machine':Machine, 'TGCC':TGCC,'IDRIS':IDRIS, 'CPU':CPU,
                    'Program'  : "Generated by : " + str(sys.argv), 
                    'HOSTNAME' : platform.node (), 'LOGNAME'  : os.getlogin (),
                    'Python'   : f'{platform.python_version ()}',
                    'OS'       : f'{platform.system ()} release : {platform.release ()} hardware : {platform.machine ()}',
                    'SVN_Author'   : "$Author$", 
                    'SVN_Date'     : "$Date$",
                    'SVN_Revision' : "$Revision$",
                    'SVN_Id'       : "$Id$",
                    'SVN_HeadURL'  : "$HeadURL$"}

if libIGCM :
    config['libIGCM'] = {'ARCHIVE':ARCHIVE, 'STORAGE':STORAGE, 'SCRATCHDIR':SCRATCHDIR, 'R_IN':R_IN, 'rebuild':rebuild }

## Import specific module
import nemo, lmdz
## Now import needed scientific modules
import xarray as xr

# Output file
if FileOut == None : 
    FileOut = f'ATM_waterbudget_{JobName}_{YearBegin}_{YearEnd}.out'
    config['Files']['FileOut'] = FileOut

f_out = open ( FileOut, mode = 'w' )

# Function to print to stdout *and* output file
def echo (string, end='\n') :
    print ( str(string), end=end  )
    sys.stdout.flush ()
    f_out.write ( str(string) + end )
    f_out.flush ()
    return None
    
## Set libIGCM directories
R_OUT       = os.path.join ( ARCHIVE   , 'IGCM_OUT')
R_BUF       = os.path.join ( SCRATCHDIR, 'IGCM_OUT')

L_EXP = os.path.join (TagName, SpaceName, ExperimentName, JobName)
R_SAVE      = os.path.join ( R_OUT, L_EXP )
R_BUFR      = os.path.join ( R_BUF, L_EXP )
POST_DIR    = os.path.join ( R_BUF, L_EXP, 'Out' )
REBUILD_DIR = os.path.join ( R_BUF, L_EXP, 'REBUILD' )
R_BUF_KSH   = os.path.join ( R_BUFR, 'Out' )
R_FIGR      = os.path.join ( STORAGE, 'IGCM_OUT', L_EXP )

#if os.path.isdir (TmpDir) : shutil.rmtree ( TmpDir )
if not os.path.isdir (TmpDir) : os.mkdir (TmpDir)
TmpDirOCE = os.path.join (TmpDir, 'OCE')
TmpDirICE = os.path.join (TmpDir, 'ICE')
if not os.path.exists (TmpDirOCE) : os.mkdir (TmpDirOCE )
if not os.path.exists (TmpDirICE) : os.mkdir (TmpDirICE )

echo (' ')
echo ( f'JobName : {JobName}' )
echo (Comment)
echo ( f'Working in TMPDIR : {TmpDir}' )

echo ( f'\nDealing with {L_EXP}' )

#-- Model output directories
if Freq == "MO" : FreqDir = os.path.join ('Output' , 'MO' )
if Freq == "SE" : FreqDir = os.path.join ('Analyse', 'SE' )
if dir_ATM_his == None :
    dir_ATM_his = os.path.join ( R_SAVE, "ATM", FreqDir )
    config['Files']['dir_ATM_his'] = dir_ATM_his
if dir_SRF_his == None : 
    dir_SRF_his = os.path.join ( R_SAVE, "SRF", FreqDir )
    config['Files']['dir_SRF_his'] = dir_SRF_his
    
echo ( f'The analysis relies on files from the following model output directories : ' )
echo ( f'{dir_ATM_his}' )
echo ( f'{dir_SRF_his}' )

#-- Files Names
if Period == None :
    if Freq == 'MO' : Period = f'{YearBegin}0101_{YearEnd}1231_1M'
    if Freq == 'SE' : Period = f'SE_{YearBegin}0101_{YearEnd}1231_1M'
    config['Files']['Period'] = Period
if FileCommon == None :
    FileCommon = f'{JobName}_{Period}'
    config['Files']['FileCommon'] = FileCommon

if Title == None :
    Title = f'{JobName} : {Freq} : {YearBegin}-01-01 - {YearEnd}-12-31'
    config['Files']['Title'] = Title
     
echo ('\nOpen history files' )
if file_ATM_his == None : 
    file_ATM_his = os.path.join (  dir_ATM_his, f'{FileCommon}_histmth.nc' )
    config['Files']['file_ATM_his'] = file_ATM_his
if file_SRF_his == None :
    file_SRF_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
    config['Files']['file_SRF_his'] = file_SRF_his
#if Routing == 'SECHIBA'  :
#     file_RUN_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
if Routing == 'SIMPLE' :
    if file_RUN_his == None : 
        file_RUN_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
        config['Files']['file_RUN_his'] = file_RUN_his

d_ATM_his = xr.open_dataset ( file_ATM_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_SRF_his = xr.open_dataset ( file_SRF_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if Routing == 'SECHIBA' :
    d_RUN_his = d_SRF_his
if Routing == 'SIMPLE' : 
    d_RUN_his = xr.open_dataset ( file_RUN_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

echo ( file_ATM_his )
echo ( file_SRF_his )
if Routing == 'SIMPLE' : 
    echo ( file_RUN_his )

## Compute run length
dtime = ( d_ATM_his.time_counter_bounds.max() - d_ATM_his.time_counter_bounds.min() )
echo ('\nRun length : {:8.2f} days'.format ( (dtime/np.timedelta64(1, "D")).values ) )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

## Compute length of each period
dtime_per = (d_ATM_his.time_counter_bounds[:,-1] -  d_ATM_his.time_counter_bounds[:,0] )
echo ('\nPeriods lengths (days) : {:} days'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_ATM_his.time_counter,] )

## Number of years
NbYear = dtime_sec / YearLength

#-- Open restart files

YearRes = YearBegin - 1              # Year of the restart of beginning of simulation
YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation

if TarRestartPeriod_beg == None : 
    echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')
    TarRestartPeriod_beg = f'{YearPre}0101_{YearRes}1231'
    config['Files']['TarRestartPeriod_beg'] = TarRestartPeriod_beg

if TarRestartPeriod_end == None : 
    YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation
    echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')
    TarRestartPeriod_end = f'{YearBegin}0101_{YearEnd}1231'
    config['Files']['TarRestartPeriod_end'] = TarRestartPeriod_end

if file_restart_beg == None :
    file_restart_beg = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_beg}_restart.tar' )
    config['Files']['file_restart_beg'] = file_restart_beg
if file_restart_end == None :
    file_restart_end = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_end}_restart.tar' )
    config['Files']['file_restart_end'] = file_restart_end
    
echo ( f'{file_restart_beg}' )
echo ( f'{file_restart_end}' )

if file_ATM_beg == None : 
    file_ATM_beg = f'{TmpDir}/ATM_{JobName}_{YearRes}1231_restartphy.nc'
    config['Files']['file_ATM_beg'] = file_ATM_beg
if file_ATM_end == None :
    file_ATM_end = f'{TmpDir}/ATM_{JobName}_{YearEnd}1231_restartphy.nc'
    config['Files']['file_ATM_end'] = file_ATM_end

liste_beg = [file_ATM_beg, ]
liste_end = [file_ATM_end, ]
    

if file_DYN_beg == None :
    if LMDZ :
        file_DYN_beg = f'{TmpDir}/ATM_{JobName}_{YearRes}1231_restart.nc'
    if ICO  :
        file_DYN_beg = f'{TmpDir}/ICO_{JobName}_{YearRes}1231_restart.nc'
    liste_beg.append (file_DYN_beg)
    config['Files']['file_DYN_beg'] = file_DYN_beg
    
if file_DYN_end == None : 
    if LMDZ :
        file_DYN_end = f'{TmpDir}/ATM_{JobName}_{YearEnd}1231_restart.nc'
    if ICO  :
        file_DYN_end = f'{TmpDir}/ICO_{JobName}_{YearEnd}1231_restart.nc'
    liste_end.append ( file_DYN_end )
    config['Files']['file_DYN_end'] = file_DYN_end

if file_SRF_beg == None : 
    file_SRF_beg = f'{TmpDir}/SRF_{JobName}_{YearRes}1231_sechiba_rest.nc'
    config['Files']['file_SRF_beg'] = file_SRF_beg
if file_SRF_end == None : 
    file_SRF_end = f'{TmpDir}/SRF_{JobName}_{YearEnd}1231_sechiba_rest.nc'
    config['Files']['file_SRF_end'] = file_SRF_end
    
liste_beg.append ( file_SRF_beg )
liste_end.append ( file_SRF_end )

echo ( f'{file_ATM_beg}')
echo ( f'{file_ATM_end}')
echo ( f'{file_DYN_beg}')
echo ( f'{file_DYN_end}')
echo ( f'{file_SRF_beg}')
echo ( f'{file_SRF_end}')

if Routing == 'SIMPLE' :
    if file_RUN_beg == None : 
        file_RUN_beg = f'{TmpDir}/SRF_{JobName}_{YearRes}1231_routing_restart.nc'
        config['Files']['file_RUN_beg'] = file_RUN_beg
    if file_RUN_end == None : 
        file_RUN_end = f'{TmpDir}/SRF_{JobName}_{YearEnd}1231_routing_restart.nc'
        config['Files']['file_RUN_end'] = file_RUN_end
        
    liste_beg.append ( file_RUN_beg )
    liste_end.append ( file_RUN_end )
    echo ( f'{file_RUN_beg}')
    echo ( f'{file_RUN_end}')

echo ('\nExtract restart files from tar : ATM, ICO and SRF')
for resFile in liste_beg :
    if os.path.exists ( os.path.join (TmpDir, resFile) ) :
        echo ( f'file found : {resFile}' )
    else :
        base_resFile = os.path.basename (resFile)
        command =  f'cd {TmpDir} ; tar xf {file_restart_beg} {base_resFile}'
        echo ( command )
        os.system ( command )
        
for resFile in liste_end :
    if os.path.exists ( os.path.join (TmpDir, resFile) ) :
        echo ( f'file found : {resFile}' )
    else :
        base_resFile = os.path.basename (resFile)
        command =  f'cd {TmpDir} ; tar xf {file_restart_end} {base_resFile}'
        echo ( command )
        os.system ( command )
       
echo ('\nOpening ATM SRF and ICO restart files')
d_ATM_beg = xr.open_dataset ( os.path.join (TmpDir, file_ATM_beg), decode_times=False, decode_cf=True).squeeze()
d_ATM_end = xr.open_dataset ( os.path.join (TmpDir, file_ATM_end), decode_times=False, decode_cf=True).squeeze()
d_SRF_beg = xr.open_dataset ( os.path.join (TmpDir, file_SRF_beg), decode_times=False, decode_cf=True).squeeze()
d_SRF_end = xr.open_dataset ( os.path.join (TmpDir, file_SRF_end), decode_times=False, decode_cf=True).squeeze()
d_DYN_beg = xr.open_dataset ( os.path.join (TmpDir, file_DYN_beg), decode_times=False, decode_cf=True).squeeze()
d_DYN_end = xr.open_dataset ( os.path.join (TmpDir, file_DYN_end), decode_times=False, decode_cf=True).squeeze()

for var in d_SRF_beg.variables :
    d_SRF_beg[var] = d_SRF_beg[var].where ( d_SRF_beg[var]<1.e20, 0.)
    d_SRF_end[var] = d_SRF_end[var].where ( d_SRF_end[var]<1.e20, 0.)

if Routing == 'SIMPLE' :
    d_RUN_beg = xr.open_dataset ( os.path.join (TmpDir, file_RUN_beg), decode_times=False, decode_cf=True).squeeze()
    d_RUN_end = xr.open_dataset ( os.path.join (TmpDir, file_RUN_end), decode_times=False, decode_cf=True).squeeze()
    
echo ( file_ATM_beg )
echo ( file_ATM_end )
echo ( file_DYN_beg )
echo ( file_DYN_end )
echo ( file_SRF_beg )
echo ( file_SRF_end )
if Routing == 'SIMPLE' :
    echo ( file_RUN_beg )
    echo ( file_RUN_end )

##
config_out = open (FullIniFile, 'w')
config.write (config_out )
config_out.close ()

def Ksum (tab) :
    '''
    Kahan summation algorithm, also known as compensated summation
    https://en.wikipedia.org/wiki/Kahan_summation_algorithm
    '''
    Ksum = 0.0                   # Prepare the accumulator.
    comp = 0.0                   # A running compensation for lost low-order bits.
   
    for xx in np.where ( np.isnan(tab), 0., tab ) : 
        yy = xx - comp           # comp is zero the first time around.
        tt = Ksum + yy           # Alas, sum is big, y small, so low-order digits of y are lost.
        comp = (tt - Ksum) - yy  # (tt - Ksum) cancels the high-order part of y; subtracting y recovers negative (low part of yy)
        Ksum = tt                # Algebraically, comp should always be zero. Beware overly-aggressive optimizing compilers!
                                 # Next time around, the lost low part will be added to y in a fresh attempt.
    return Ksum

def Ssum (tab) :
    '''
    Precision summation by sorting by absolute value
    '''
    Ssum = np.sum ( tab[np.argsort(np.abs(tab))] )
    return Ssum

def KSsum (tab) :
    '''
    Precision summation by sorting by absolute value, and applying Kahan summation
    '''
    KSsum = Ksum ( tab[np.argsort(np.abs(tab))] )
    return KSsum

# Choosing algorithm
Psum = KSsum

def ATM_stock_int (stock) :
    '''Integrate stock on atmosphere grid'''
    ATM_stock_int  = Psum ( (stock * DYN_aire).to_masked_array().ravel() ) 
    return ATM_stock_int

def ATM_flux_int (flux) :
    '''Integrate flux on atmosphere grid'''
    ATM_flux_int  = Psum ( (flux * dtime_per_sec * ATM_aire).to_masked_array().ravel() )
    return ATM_flux_int

def SRF_stock_int (stock) :
    '''Integrate stock on land grid'''
    ATM_stock_int  = Ksum (  ( (stock * DYN_aire_fter).to_masked_array().ravel()) )
    return ATM_stock_int

def SRF_flux_int (flux) :
    '''Integrate flux on land grid'''
    SRF_flux_int  = Psum (  (flux * dtime_per_sec * SRF_aire).to_masked_array().ravel() )
    return SRF_flux_int

def ONE_stock_int (stock) :
    '''Sum stock '''
    ONE_stock_int  = Psum ( stock.to_masked_array().ravel() )
    return ONE_stock_int

def ONE_flux_int (flux) :
    '''Sum flux '''
    ONE_flux_int  = Psum ( (flux * dtime_per_sec ).to_masked_array().ravel() )
    return ONE_flux_int

def kg2Sv  (val, rho=ATM_rho) :
    '''From kg to Sverdrup'''
    return val/dtime_sec*1.0e-6/rho

def kg2myear (val, rho=ATM_rho) :
    '''From kg to m/year'''
    return val/ATM_aire_sea_tot/rho/NbYear
    
# ATM grid with cell surfaces
if ICO :
    jpja, jpia = d_ATM_his['aire'][0].shape   
    file_ATM_aire = os.path.join ( R_IN, 'ATM', 'GRID', f'aire_{ATM}_to_{jpia}x{jpja}.nc' )
    config['Files']['file_ATM_aire'] = file_ATM_aire
    echo ( f'Aire sur grille reguliere : {file_ATM_aire}' )
    d_ATM_aire = xr.open_dataset ( file_ATM_aire, decode_times=False ).squeeze()
    ATM_aire = lmdz.geo2point ( d_ATM_aire ['aire'].squeeze(), cumulPoles=True )
    ATM_fter = lmdz.geo2point ( d_ATM_his ['fract_ter'][0] )
    ATM_foce = lmdz.geo2point ( d_ATM_his ['fract_oce'][0] )
    ATM_fsic = lmdz.geo2point ( d_ATM_his ['fract_sic'][0] )
    ATM_flic = lmdz.geo2point ( d_ATM_his ['fract_lic'][0] )
    ATM_fsea = ATM_foce + ATM_fsic
    ATM_flnd = ATM_fter + ATM_flic
    SRF_aire = lmdz.geo2point ( d_SRF_his ['Areas'] * d_SRF_his ['Contfrac'] )
    #SRF_aire = ATM_aire * lmdz.geo2point (d_SRF_his ['Contfrac'] )
    #SRF_aire = ATM_aire * ATM_fter
    
if LMDZ :
    ATM_aire = lmdz.geo2point ( d_ATM_his ['aire'][0], cumulPoles=True )
    ATM_fsea = lmdz.geo2point ( d_ATM_his ['fract_oce'][0] + d_ATM_his ['fract_sic'][0] )
    ATM_flnd = lmdz.geo2point ( d_ATM_his ['fract_ter'][0] )
    #SRF_aire = lmdz.geo2point ( d_SRF_his['Areas'] * d_SRF_his['Contfrac'] )
    SRF_aire = ATM_aire * lmdz.geo2point ( d_SRF_his['Contfrac'] )

SRF_aire = SRF_aire.where ( SRF_aire < 1E15, 0.)

if ICO :
    # Area on icosahedron grid
    file_DYN_aire = os.path.join ( R_IN, 'ATM', 'GRID', ATM+'_grid.nc' )
    d_DYN_aire = xr.open_dataset ( file_DYN_aire, decode_times=False).squeeze()
    d_DYN_aire = d_DYN_aire.rename ( {'cell':'cell_mesh'} )
    DYN_aire   = d_DYN_aire['aire']

    DYN_fsea = d_DYN_aire ['fract_oce'] + d_DYN_aire ['fract_sic']
    DYN_flnd = 1.0 - DYN_fsea
    DYN_fter = d_ATM_beg['FTER'].rename({'points_physiques':'cell_mesh'})
    DYN_flic = d_ATM_beg['FTER'].rename({'points_physiques':'cell_mesh'})
    
if LMDZ :
    DYN_aire = ATM_aire
    DYN_fsea = ATM_fsea
    DYN_flnd = ATM_flnd
    DYN_fter = d_ATM_beg['FTER']
    DYN_flic = d_ATM_beg['FTER']

    
DYN_aire_fter = DYN_aire * DYN_fter

#if LMDZ :
#    d_ATM_beg = d_ATM_beg.assign ( coords={'lon':d_ATM_beg.lon*180./np.pi} )

ATM_aire_sea     = ATM_aire * ATM_fsea
ATM_aire_sea_tot = ONE_stock_int ( ATM_aire_sea )

ATM_aire_tot     = ONE_stock_int (ATM_aire)
ATM_aire_sea_tot = ONE_stock_int (ATM_aire*ATM_fsea)


echo ( 'Aire atmosphere/4pi R^2 : {:12.5f}'.format(ATM_aire_tot/(Ra*Ra*4*np.pi) ) )

if (  np.abs (ATM_aire_tot/(Ra*Ra*4*np.pi) - 1.0) > 0.01 ) :
    raise Exception ('Erreur surface interpolee sur grille reguliere')

echo ( '\n====================================================================================' )
echo ( f'-- ATM changes in stores  -- {Title} ' )

#-- Change in precipitable water from the atmosphere daily and monthly files
#-- Compute sum weighted by gridcell area (kg/m2) then convert to Sv

# ATM vertical grid
ATM_Ahyb = d_ATM_his['Ahyb'].squeeze()
ATM_Bhyb = d_ATM_his['Bhyb'].squeeze()

# Surface pressure
if ICO : 
    DYN_psol_beg = d_DYN_beg['ps']
    DYN_psol_end = d_DYN_end['ps']
if LMDZ : 
    DYN_psol_beg = lmdz.geo2point ( d_DYN_beg['ps'].isel(rlonv=slice(0,-1)) )
    DYN_psol_end = lmdz.geo2point ( d_DYN_end['ps'].isel(rlonv=slice(0,-1)) )
    
# 3D Pressure at the interface layers (not scalar points)
DYN_pres_beg = ATM_Ahyb + ATM_Bhyb * DYN_psol_beg
DYN_pres_end = ATM_Ahyb + ATM_Bhyb * DYN_psol_end

klevp1 = ATM_Bhyb.shape[-1]
if ICO  : cell_mesh        = DYN_psol_beg.shape[-1]
if LMDZ : points_physiques = DYN_psol_beg.shape[-1]
klev = klevp1 - 1

# Layer thickness (pressure)
if ICO : 
    DYN_dpres_beg = xr.DataArray ( np.empty( (klev, cell_mesh       )), dims=('sigs', 'cell_mesh'       ), coords=(np.arange(klev), np.arange(cell_mesh)        ) )
    DYN_dpres_end = xr.DataArray ( np.empty( (klev, cell_mesh       )), dims=('sigs', 'cell_mesh'       ), coords=(np.arange(klev), np.arange(cell_mesh)        ) )
if LMDZ : 
    DYN_dpres_beg = xr.DataArray ( np.empty( (klev, points_physiques)), dims=('sigs', 'points_physiques'), coords=(np.arange(klev), np.arange(points_physiques) ) )
    DYN_dpres_end = xr.DataArray ( np.empty( (klev, points_physiques)), dims=('sigs', 'points_physiques'), coords=(np.arange(klev), np.arange(points_physiques) ) )

for k in np.arange (klevp1-1) :
    DYN_dpres_beg[k,:] = DYN_pres_beg[k,:] - DYN_pres_beg[k+1,:]
    DYN_dpres_end[k,:] = DYN_pres_end[k,:] - DYN_pres_end[k+1,:]

##-- Vertical and horizontal integral, and sum of liquid, solid and vapor water phases
if LMDZ :
    try : 
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2Ov']  + d_DYN_beg['H2Ol']  + d_DYN_beg['H2Oi'] ).isel(rlonv=slice(0,-1) ) )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2Ov']  + d_DYN_end['H2Ol']  + d_DYN_end['H2Oi'] ).isel(rlonv=slice(0,-1) ) )
    except :
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).isel(rlonv=slice(0,-1) ) )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).isel(rlonv=slice(0,-1) ) )
if ICO :
    try :
        DYN_wat_beg = (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).rename ( {'lev':'sigs'} )
        DYN_wat_end = (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).rename ( {'lev':'sigs'} )
    except : 
        DYN_wat_beg = (d_DYN_beg['q'].isel(nq=0) + d_DYN_beg['q'].isel(nq=1) + d_DYN_beg['q'].isel(nq=2) ).rename ( {'lev':'sigs'} )
        DYN_wat_end = (d_DYN_end['q'].isel(nq=0) + d_DYN_end['q'].isel(nq=1) + d_DYN_end['q'].isel(nq=2) ).rename ( {'lev':'sigs'} )
    
# Compute water content : vertical and horizontal integral
DYN_mas_wat_beg = ATM_stock_int (DYN_dpres_beg * DYN_wat_beg) / Grav
DYN_mas_wat_end = ATM_stock_int (DYN_dpres_end * DYN_wat_end) / Grav

# Variation of water content
dDYN_mas_wat = DYN_mas_wat_end - DYN_mas_wat_beg

# \([a-z,A-Z,_]*\)/dtime_sec\*1e-9  kg2Sv(\1)
# \([a-z,A-Z,_]*\)\/ATM_aire_sea_tot\/ATM_rho\/NbYear  kg2myear(\1)

def var2prt (var, small=False) :
    if small :  return var , kg2Sv(var)*1000., kg2myear(var)*1000.
    else     :  return var , kg2Sv(var)      , kg2myear(var)

def prtFlux (Desc, var, Form='F', small=False) :
    if small :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>15} = ' +ff).format (Desc, *var2prt(var, small) ) )
    return None

echo ( f'\nVariation du contenu en eau atmosphere (dynamique)  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'DYN_mas_beg = {:12.6e} kg | DYN_mas_end = {:12.6e} kg'.format (DYN_mas_wat_beg, DYN_mas_wat_end) )
prtFlux ( 'dMass (atm)  ', dDYN_mas_wat, 'e', True )

ATM_sno_beg = d_ATM_beg['SNOW01']*d_ATM_beg['FTER']+d_ATM_beg['SNOW02']*d_ATM_beg['FLIC']+d_ATM_beg['SNOW03']*d_ATM_beg['FOCE']+d_ATM_beg['SNOW04']*d_ATM_beg['FSIC']
ATM_sno_end = d_ATM_end['SNOW01']*d_ATM_end['FTER']+d_ATM_end['SNOW02']*d_ATM_end['FLIC']+d_ATM_end['SNOW03']*d_ATM_end['FOCE']+d_ATM_end['SNOW04']*d_ATM_end['FSIC']

ATM_qs_beg  = d_ATM_beg['QS01']*d_ATM_beg['FTER']+d_ATM_beg['QS02']*d_ATM_beg['FLIC']+d_ATM_beg['QS03']*d_ATM_beg['FOCE']+d_ATM_beg['QS04']*d_ATM_beg['FSIC']
ATM_qs_end  = d_ATM_end['QS01']*d_ATM_end['FTER']+d_ATM_end['QS02']*d_ATM_end['FLIC']+d_ATM_end['QS03']*d_ATM_end['FOCE']+d_ATM_end['QS04']*d_ATM_end['FSIC']

ATM_qsol_beg = d_ATM_beg['QSOL']
ATM_qsol_end = d_ATM_end['QSOL']

ATM_qs01_beg  = d_ATM_beg['QS01'] * d_ATM_beg['FTER']
ATM_qs01_end  = d_ATM_end['QS01'] * d_ATM_end['FTER']
ATM_qs02_beg  = d_ATM_beg['QS02'] * d_ATM_beg['FLIC']
ATM_qs02_end  = d_ATM_end['QS02'] * d_ATM_end['FLIC']
ATM_qs03_beg  = d_ATM_beg['QS03'] * d_ATM_beg['FOCE']
ATM_qs03_end  = d_ATM_end['QS03'] * d_ATM_end['FOCE']
ATM_qs04_beg  = d_ATM_beg['QS04'] * d_ATM_beg['FSIC']
ATM_qs04_end  = d_ATM_end['QS04'] * d_ATM_end['FSIC']  

if ICO :
   ATM_sno_beg   = ATM_sno_beg .rename ( {'points_physiques':'cell_mesh'} )
   ATM_sno_end   = ATM_sno_end .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs_beg    = ATM_qs_beg  .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs_end    = ATM_qs_end  .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qsol_beg  = ATM_qsol_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qsol_end  = ATM_qsol_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs01_beg  = ATM_qs01_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs01_end  = ATM_qs01_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs02_beg  = ATM_qs02_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs02_end  = ATM_qs02_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs03_beg  = ATM_qs03_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs03_end  = ATM_qs03_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs04_beg  = ATM_qs04_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs04_end  = ATM_qs04_end.rename ( {'points_physiques':'cell_mesh'} ) 

ATM_mas_sno_beg   = ATM_stock_int ( ATM_sno_beg  )
ATM_mas_sno_end   = ATM_stock_int ( ATM_sno_end  )
ATM_mas_qs_beg    = ATM_stock_int ( ATM_qs_beg   )
ATM_mas_qs_end    = ATM_stock_int ( ATM_qs_end   )
ATM_mas_qsol_beg  = ATM_stock_int ( ATM_qsol_beg )
ATM_mas_qsol_end  = ATM_stock_int ( ATM_qsol_end )
ATM_mas_qs01_beg  = ATM_stock_int ( ATM_qs01_beg )
ATM_mas_qs01_end  = ATM_stock_int ( ATM_qs01_end )
ATM_mas_qs02_beg  = ATM_stock_int ( ATM_qs02_beg )
ATM_mas_qs02_end  = ATM_stock_int ( ATM_qs02_end )
ATM_mas_qs03_beg  = ATM_stock_int ( ATM_qs03_beg )
ATM_mas_qs03_end  = ATM_stock_int ( ATM_qs03_end )
ATM_mas_qs04_beg  = ATM_stock_int ( ATM_qs04_beg )
ATM_mas_qs04_end  = ATM_stock_int ( ATM_qs04_end )

dATM_mas_sno  = ATM_mas_sno_end  - ATM_mas_sno_beg
dATM_mas_qs   = ATM_mas_qs_end   - ATM_mas_qs_beg
dATM_mas_qsol = ATM_mas_qsol_end - ATM_mas_qsol_beg

dATM_mas_qs01 = ATM_mas_qs01_end - ATM_mas_qs01_beg
dATM_mas_qs02 = ATM_mas_qs02_end - ATM_mas_qs02_beg
dATM_mas_qs03 = ATM_mas_qs03_end - ATM_mas_qs03_beg
dATM_mas_qs04 = ATM_mas_qs04_end - ATM_mas_qs04_beg

dATM_mas_sno_2 = ATM_stock_int ( ATM_sno_end - ATM_sno_beg )

echo ( f'\nVariation du contenu en neige atmosphere (calottes)  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_sno_beg  = {:12.6e} kg | ATM_mas_sno_end  = {:12.6e} kg'.format (ATM_mas_sno_beg, ATM_mas_sno_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_sno  , 'e', True  )
prtFlux ( 'dMass (neige atm) ', dATM_mas_sno_2, 'e', True  )

echo ( f'\nVariation du contenu humidite du sol  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'ATM_mas_qs_beg  = {:12.6e} kg | ATM_mas_qs_end  = {:12.6e} kg'.format (ATM_mas_qs_beg, ATM_mas_qs_end) )
prtFlux ( 'dMass (neige atm) ', dATM_mas_qs, 'e', True )

echo ( f'\nVariation du contenu en eau+neige atmosphere  -- {Title}  ' )
echo ( '------------------------------------------------------------------------------------' )
prtFlux ( 'dMass (eau + neige atm) ', dDYN_mas_wat + dATM_mas_sno , 'e', True)

echo ( '\n====================================================================================' )
echo ( f'-- SRF changes  -- {Title} ' )

if Routing == 'SIMPLE' :
    RUN_mas_wat_fast_beg   = ONE_stock_int ( d_RUN_beg ['fast_reservoir']   )
    RUN_mas_wat_slow_beg   = ONE_stock_int ( d_RUN_beg ['slow_reservoir']   )
    RUN_mas_wat_stream_beg = ONE_stock_int ( d_RUN_beg ['stream_reservoir'] )
    RUN_mas_wat_flood_beg  = 0.0
    RUN_mas_wat_lake_beg   = 0.0
    RUN_mas_wat_pond_beg   = 0.0
    
    RUN_mas_wat_fast_end   = ONE_stock_int ( d_RUN_end ['fast_reservoir']   )
    RUN_mas_wat_slow_end   = ONE_stock_int ( d_RUN_end ['slow_reservoir']   )
    RUN_mas_wat_stream_end = ONE_stock_int ( d_RUN_end ['stream_reservoir'] )
    RUN_mas_wat_flood_end  = 0.0
    RUN_mas_wat_lake_end   = 0.0
    RUN_mas_wat_pond_end   = 0.0

if Routing == 'SECHIBA' :
    RUN_mas_wat_fast_beg   = ONE_stock_int ( d_SRF_beg ['fastres']   )
    RUN_mas_wat_slow_beg   = ONE_stock_int ( d_SRF_beg ['slowres']   )
    RUN_mas_wat_stream_beg = ONE_stock_int ( d_SRF_beg ['streamres'] )
    RUN_mas_wat_flood_beg  = ONE_stock_int ( d_SRF_beg ['floodres']  )
    RUN_mas_wat_lake_beg   = ONE_stock_int ( d_SRF_beg ['lakeres']   )
    RUN_mas_wat_pond_beg   = ONE_stock_int ( d_SRF_beg ['pondres']   )

    
    RUN_mas_wat_fast_end   = ONE_stock_int ( d_SRF_end ['fastres']   )
    RUN_mas_wat_slow_end   = ONE_stock_int ( d_SRF_end ['slowres']   )
    RUN_mas_wat_stream_end = ONE_stock_int ( d_SRF_end ['streamres'] )
    RUN_mas_wat_flood_end  = ONE_stock_int ( d_SRF_end ['floodres']  )
    RUN_mas_wat_lake_end   = ONE_stock_int ( d_SRF_end ['lakeres']   )
    RUN_mas_wat_pond_end   = ONE_stock_int ( d_SRF_end ['pondres']   )

RUN_mas_wat_beg = RUN_mas_wat_fast_beg + RUN_mas_wat_slow_beg + RUN_mas_wat_stream_beg + RUN_mas_wat_flood_beg + RUN_mas_wat_lake_beg + RUN_mas_wat_pond_beg
RUN_mas_wat_end = RUN_mas_wat_fast_end + RUN_mas_wat_slow_end + RUN_mas_wat_stream_end + RUN_mas_wat_flood_end + RUN_mas_wat_lake_end + RUN_mas_wat_pond_end

dRUN_mas_wat_fast   = RUN_mas_wat_fast_end   - RUN_mas_wat_fast_beg
dRUN_mas_wat_slow   = RUN_mas_wat_slow_end   - RUN_mas_wat_slow_beg
dRUN_mas_wat_stream = RUN_mas_wat_stream_end - RUN_mas_wat_stream_beg
dRUN_mas_wat_flood  = RUN_mas_wat_flood_end  - RUN_mas_wat_flood_beg
dRUN_mas_wat_lake   = RUN_mas_wat_lake_end   - RUN_mas_wat_lake_beg
dRUN_mas_wat_pond   = RUN_mas_wat_pond_end   - RUN_mas_wat_pond_beg

dRUN_mas_wat        = RUN_mas_wat_end        - RUN_mas_wat_beg

echo ( f'\nLes differents reservoirs  -- {Title} ')
echo ( '------------------------------------------------------------------------------------' )
echo ( 'RUN_mas_wat_fast_beg   = {:12.6e} kg | RUN_mas_wat_fast_end   = {:12.6e} kg '.format (RUN_mas_wat_fast_beg  , RUN_mas_wat_fast_end   ) )
echo ( 'RUN_mas_wat_slow_beg   = {:12.6e} kg | RUN_mas_wat_slow_end   = {:12.6e} kg '.format (RUN_mas_wat_slow_beg  , RUN_mas_wat_slow_end   ) )
echo ( 'RUN_mas_wat_stream_beg = {:12.6e} kg | RUN_mas_wat_stream_end = {:12.6e} kg '.format (RUN_mas_wat_stream_beg, RUN_mas_wat_stream_end ) )
echo ( 'RUN_mas_wat_flood_beg  = {:12.6e} kg | RUN_mas_wat_flood_end  = {:12.6e} kg '.format (RUN_mas_wat_flood_beg , RUN_mas_wat_flood_end  ) )
echo ( 'RUN_mas_wat_lake_beg   = {:12.6e} kg | RUN_mas_wat_lake_end   = {:12.6e} kg '.format (RUN_mas_wat_lake_beg  , RUN_mas_wat_lake_end   ) )
echo ( 'RUN_mas_wat_pond_beg   = {:12.6e} kg | RUN_mas_wat_pond_end   = {:12.6e} kg '.format (RUN_mas_wat_pond_beg  , RUN_mas_wat_pond_end   ) )

echo ( '------------------------------------------------------------------------------------' )

prtFlux ( 'dMass (fast)   ', dRUN_mas_wat_fast  , 'e', True )
prtFlux ( 'dMass (slow)   ', dRUN_mas_wat_slow  , 'e', True )
prtFlux ( 'dMass (stream) ', dRUN_mas_wat_stream, 'e', True )
prtFlux ( 'dMass (flood)  ', dRUN_mas_wat_flood , 'e', True )
prtFlux ( 'dMass (lake)   ', dRUN_mas_wat_lake  , 'e', True )
prtFlux ( 'dMass (pond)   ', dRUN_mas_wat_pond  , 'e', True )
prtFlux ( 'dMass (all)    ', dRUN_mas_wat       , 'e', True )

echo ( f'\nWater content in routing  -- {Title} ' )
echo ( '------------------------------------------------------------------------------------' )
echo ( 'RUN_mas_wat_beg = {:12.6e} kg | RUN_mas_wat_end = {:12.6e} kg '.format (RUN_mas_wat_end, RUN_mas_wat_end) )
prtFlux ( 'dMass (routing) ', dRUN_mas_wat       )

echo ( '\n====================================================================================' )
print (f'Reading SRF restart')
SRF_tot_watveg_beg  = d_SRF_beg['tot_watveg_beg']  ; SRF_tot_watveg_beg  = SRF_tot_watveg_beg .where (SRF_tot_watveg_beg < 1E15, 0.)
SRF_tot_watsoil_beg = d_SRF_beg['tot_watsoil_beg'] ; SRF_tot_watsoil_beg = SRF_tot_watsoil_beg.where (SRF_tot_watsoil_beg< 1E15, 0.)
SRF_snow_beg        = d_SRF_beg['snow_beg']        ; SRF_snow_beg        = SRF_snow_beg       .where (SRF_snow_beg       < 1E15, 0.)
SRF_lakeres_beg     = d_SRF_beg['lakeres']         ; SRF_lakeres_beg     = SRF_lakeres_beg    .where (SRF_lakeres_beg    < 1E15, 0.)

SRF_tot_watveg_end  = d_SRF_end['tot_watveg_beg']  ; SRF_tot_watveg_end  = SRF_tot_watveg_end .where (SRF_tot_watveg_end < 1E15, 0.)
SRF_tot_watsoil_end = d_SRF_end['tot_watsoil_beg'] ; SRF_tot_watsoil_end = SRF_tot_watsoil_end.where (SRF_tot_watsoil_end< 1E15, 0.)
SRF_snow_end        = d_SRF_end['snow_beg']        ; SRF_snow_end        = SRF_snow_end       .where (SRF_snow_end       < 1E15, 0.)
SRF_lakeres_end     = d_SRF_end['lakeres']         ; SRF_lakeres_end     = SRF_lakeres_end    .where (SRF_lakeres_end    < 1E15, 0.)

if LMDZ :
    SRF_tot_watveg_beg  = lmdz.geo2point (SRF_tot_watveg_beg )
    SRF_tot_watsoil_beg = lmdz.geo2point (SRF_tot_watsoil_beg)
    SRF_snow_beg        = lmdz.geo2point (SRF_snow_beg       )
    SRF_lakeres_beg     = lmdz.geo2point (SRF_lakeres_beg    )
    SRF_tot_watveg_end  = lmdz.geo2point (SRF_tot_watveg_end )
    SRF_tot_watsoil_end = lmdz.geo2point (SRF_tot_watsoil_end)
    SRF_snow_end        = lmdz.geo2point (SRF_snow_end       )
    SRF_lakeres_end     = lmdz.geo2point (SRF_lakeres_end    )
  
if ICO :
    SRF_tot_watveg_beg  = SRF_tot_watveg_beg .rename ( {'y':'cell_mesh'} )
    SRF_tot_watsoil_beg = SRF_tot_watsoil_beg.rename ( {'y':'cell_mesh'} )
    SRF_snow_beg        = SRF_snow_beg       .rename ( {'y':'cell_mesh'} )
    SRF_lakeres_beg     = SRF_lakeres_beg    .rename ( {'y':'cell_mesh'} )
    SRF_tot_watveg_end  = SRF_tot_watveg_end .rename ( {'y':'cell_mesh'} )
    SRF_tot_watsoil_end = SRF_tot_watsoil_end.rename ( {'y':'cell_mesh'} )
    SRF_snow_end        = SRF_snow_end       .rename ( {'y':'cell_mesh'} )
    SRF_lakeres_end     = SRF_lakeres_end    .rename ( {'y':'cell_mesh'} )

# Stock dSoilHum dInterce dSWE dStream dFastR dSlowR dLake dPond dFlood

SRF_wat_beg = SRF_tot_watveg_beg + SRF_tot_watsoil_beg + SRF_snow_beg 
SRF_wat_end = SRF_tot_watveg_end + SRF_tot_watsoil_end + SRF_snow_end

echo ( '\n====================================================================================' )
print ('Computing integrals')

print ( ' 1/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watveg_beg   = SRF_stock_int ( SRF_tot_watveg_beg  )
print ( ' 2/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_beg  = SRF_stock_int ( SRF_tot_watsoil_beg )
print ( ' 3/8', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_beg     = SRF_stock_int ( SRF_snow_beg        )
print ( ' 4/8', end='' ) ; sys.stdout.flush ()
SRF_mas_lake_beg     = ONE_stock_int ( SRF_lakeres_beg     )
print ( ' 5/8', end='' ) ; sys.stdout.flush ()

SRF_mas_watveg_end   = SRF_stock_int ( SRF_tot_watveg_end  )
print ( ' 6/8', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_end  = SRF_stock_int ( SRF_tot_watsoil_end )
print ( ' 7/8', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_end     = SRF_stock_int ( SRF_snow_end        )
print ( ' 8/8', end='' ) ; sys.stdout.flush ()
SRF_mas_lake_end     = ONE_stock_int ( SRF_lakeres_end     )

print (' -- ') ; sys.stdout.flush ()

dSRF_mas_watveg   = SRF_mas_watveg_end  - SRF_mas_watveg_beg
dSRF_mas_watsoil  = SRF_mas_watsoil_end - SRF_mas_watsoil_beg
dSRF_mas_snow     = SRF_mas_snow_end    - SRF_mas_snow_beg
dSRF_mas_lake     = SRF_mas_lake_end    - SRF_mas_lake_beg



echo ( '------------------------------------------------------------------------------------' )
echo ( f'\nLes differents reservoirs  -- {Title} ')
echo ( 'SRF_mas_watveg_beg   = {:12.6e} kg | SRF_mas_watveg_end   = {:12.6e} kg '.format (SRF_mas_watveg_beg  , SRF_mas_watveg_end   ) )
echo ( 'SRF_mas_watsoil_beg  = {:12.6e} kg | SRF_mas_watsoil_end  = {:12.6e} kg '.format (SRF_mas_watsoil_beg , SRF_mas_watsoil_end  ) )
echo ( 'SRF_mas_snow_beg     = {:12.6e} kg | SRF_mas_snow_end     = {:12.6e} kg '.format (SRF_mas_snow_beg    , SRF_mas_snow_end     ) )
echo ( 'SRF_mas_lake_beg     = {:12.6e} kg | SRF_mas_lake_end     = {:12.6e} kg '.format (SRF_mas_lake_beg    , SRF_mas_lake_end     ) )

prtFlux ('dMass (watveg) ', dSRF_mas_watveg , 'e' , True)
prtFlux ('dMass (watsoil)', dSRF_mas_watsoil, 'e' , True)
prtFlux ('dMass (snow)   ', dSRF_mas_snow   , 'e' , True)
prtFlux ('dMass (lake)   ', dSRF_mas_lake   , 'e' , True)

SRF_mas_wat_beg = SRF_mas_watveg_beg + SRF_mas_watsoil_beg + SRF_mas_snow_beg 
SRF_mas_wat_end = SRF_mas_watveg_end + SRF_mas_watsoil_end + SRF_mas_snow_end 
dSRF_mas_wat    = SRF_mas_wat_end - SRF_mas_wat_beg

echo ( '------------------------------------------------------------------------------------' )
echo ( f'Water content in surface  -- {Title} ' )
echo ( 'SRF_mas_wat_beg   = {:12.6e} kg | SRF_mas_wat_end  = {:12.6e} kg '.format (SRF_mas_wat_beg, SRF_mas_wat_end) )
prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', True )

echo ( '------------------------------------------------------------------------------------' )
echo ( 'Water content in  ATM + SRF + RUN + LAKE' )
echo ( 'mas_wat_beg = {:12.6e} kg | mas_wat_end = {:12.6e} kg '.
           format (DYN_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg + SRF_mas_lake_beg ,
                   DYN_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end + SRF_mas_lake_end ) )
prtFlux ( 'dMass (water atm+srf+run+lake)', dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat + dSRF_mas_lake, 'e', True)

echo ( '\n====================================================================================' )
echo ( f'-- ATM Fluxes  -- {Title} ' )

ATM_wbilo_oce   = lmdz.geo2point ( d_ATM_his ['wbilo_oce']   )
ATM_wbilo_sic   = lmdz.geo2point ( d_ATM_his ['wbilo_sic']   )
ATM_wbilo_ter   = lmdz.geo2point ( d_ATM_his ['wbilo_ter']   )
ATM_wbilo_lic   = lmdz.geo2point ( d_ATM_his ['wbilo_lic']   )
ATM_runofflic   = lmdz.geo2point ( d_ATM_his ['runofflic']   )
ATM_fqcalving   = lmdz.geo2point ( d_ATM_his ['fqcalving']   )
ATM_fqfonte     = lmdz.geo2point ( d_ATM_his ['fqfonte']     )
ATM_precip      = lmdz.geo2point ( d_ATM_his ['precip']      )
ATM_snowf       = lmdz.geo2point ( d_ATM_his ['snow']        )
ATM_evap        = lmdz.geo2point ( d_ATM_his ['evap']        )
ATM_wbilo       = ATM_wbilo_oce + ATM_wbilo_sic + ATM_wbilo_ter + ATM_wbilo_lic
ATM_emp         = ATM_evap - ATM_precip

RUN_coastalflow = lmdz.geo2point ( d_RUN_his ['coastalflow'] ) 
RUN_riverflow   = lmdz.geo2point ( d_RUN_his ['riverflow']   )
RUN_runoff      = lmdz.geo2point ( d_RUN_his ['runoff']      )
RUN_drainage    = lmdz.geo2point ( d_RUN_his ['drainage']    )
RUN_riversret   = lmdz.geo2point ( d_RUN_his ['riversret']   )

RUN_coastalflow_cpl = lmdz.geo2point ( d_RUN_his ['coastalflow_cpl'] ) 
RUN_riverflow_cpl   = lmdz.geo2point ( d_RUN_his ['riverflow_cpl']   )

SRF_rain     = lmdz.geo2point ( d_SRF_his ['rain']  )
SRF_evap     = lmdz.geo2point ( d_SRF_his ['evap']  )
SRF_snowf    = lmdz.geo2point ( d_SRF_his ['snowf'] )
SRF_subli    = lmdz.geo2point ( d_SRF_his ['subli'] )
SRF_transpir = lmdz.geo2point ( np.sum(d_SRF_his ['transpir'], axis=1) ) ; SRF_transpir.attrs['units'] = d_SRF_his ['transpir'].attrs['units']
SRF_emp      = SRF_evap - SRF_rain - SRF_snowf ; SRF_emp.attrs['units'] = SRF_rain.attrs['units']

## Correcting units of SECHIBA variables
def mmd2SI ( Var ) :
    '''Change unit from mm/d or m^3/s to kg/s if needed'''
    if 'units' in VarT.attrs : 
        if VarT.attrs['units'] in ['m^3/s', 'm3/s', 'm3.s-3'] :
            VarT.values = VarT.values  * ATM_rho                 ;  VarT.attrs['units'] = 'kg/s'
        if VarT.attrs['units'] == 'mm/d' :
            VarT.values = VarT.values  * ATM_rho * (1e-3/86400.) ;  VarT.attrs['units'] = 'kg/s'

for var in [ 'runoff', 'drainage', 'riversret', 'coastalflow', 'riverflow', 'coastalflow_cpl', 'riverflow_cpl' ] :
    VarT = locals()['RUN_' + var]
    mmd2SI (VarT)

for var in ['evap', 'snowf', 'subli', 'transpir', 'rain', 'emp' ] :
    VarT = locals()['SRF_' + var]
    mmd2SI (VarT)


RUN_input  = RUN_runoff      + RUN_drainage
RUN_output = RUN_coastalflow + RUN_riverflow

ATM_wbilo_sea = ATM_wbilo_oce + ATM_wbilo_sic

ATM_flx_oce         = ATM_flux_int ( ATM_wbilo_oce  )
ATM_flx_sic         = ATM_flux_int ( ATM_wbilo_sic  )
ATM_flx_sea         = ATM_flux_int ( ATM_wbilo_sea  )
ATM_flx_ter         = ATM_flux_int ( ATM_wbilo_ter  )
ATM_flx_lic         = ATM_flux_int ( ATM_wbilo_lic  )
ATM_flx_calving     = ATM_flux_int ( ATM_fqcalving  )
ATM_flx_qfonte      = ATM_flux_int ( ATM_fqfonte    )
ATM_flx_precip      = ATM_flux_int ( ATM_precip     )
ATM_flx_snowf       = ATM_flux_int ( ATM_snowf      )
ATM_flx_evap        = ATM_flux_int ( ATM_evap       )
ATM_flx_runlic      = ATM_flux_int ( ATM_runofflic  )

RUN_flx_coastal     = ONE_flux_int ( RUN_coastalflow)
RUN_flx_river       = ONE_flux_int ( RUN_riverflow  )
RUN_flx_coastal_cpl = ONE_flux_int ( RUN_coastalflow_cpl)
RUN_flx_river_cpl   = ONE_flux_int ( RUN_riverflow_cpl  )
RUN_flx_drainage    = SRF_flux_int ( RUN_drainage   )
RUN_flx_riversret   = SRF_flux_int ( RUN_riversret  )
RUN_flx_runoff      = SRF_flux_int ( RUN_runoff     )
RUN_flx_input       = SRF_flux_int ( RUN_input      )
RUN_flx_output      = ONE_flux_int ( RUN_output     )

ATM_flx_emp   = ATM_flux_int (ATM_emp)
ATM_flx_wbilo = ATM_flux_int (ATM_wbilo)

RUN_flx_bil   = RUN_flx_input - RUN_flx_output
RUN_flx_rivcoa = RUN_flx_coastal + RUN_flx_river

prtFlux ('wbilo_oce     ', ATM_flx_oce, 'f' )
prtFlux ('wbilo_oce     ', ATM_flx_oce, 'f' )          
prtFlux ('wbilo_sic     ', ATM_flx_sic, 'f' )          
prtFlux ('wbilo_sic+oce ', ATM_flx_sea, 'f' )          
prtFlux ('wbilo_ter     ', ATM_flx_ter, 'f' )          
prtFlux ('wbilo_lic     ', ATM_flx_lic, 'f' )          
prtFlux ('Sum wbilo_*   ', ATM_flx_wbilo      , 'f', True)  
prtFlux ('E-P           ', ATM_flx_emp        , 'f', True)  
prtFlux ('calving       ', ATM_flx_calving    , 'f' ) 
prtFlux ('fqfonte       ', ATM_flx_qfonte     , 'f' )       
prtFlux ('precip        ', ATM_flx_precip     , 'f' )       
prtFlux ('snowf         ', ATM_flx_snowf      , 'f' )        
prtFlux ('evap          ', ATM_flx_evap       , 'f' )         
prtFlux ('coastalflow   ', RUN_flx_coastal    , 'f' ) 
prtFlux ('riverflow     ', RUN_flx_river      , 'f' )        
prtFlux ('coastal_cpl   ', RUN_flx_coastal_cpl, 'f' )  
prtFlux ('riverf_cpl    ', RUN_flx_river_cpl  , 'f' )    
prtFlux ('river+coastal ', RUN_flx_rivcoa     , 'f' )   
prtFlux ('drainage      ', RUN_flx_drainage   , 'f' )   
prtFlux ('riversret     ', RUN_flx_riversret  , 'f' )   
prtFlux ('runoff        ', RUN_flx_runoff     , 'f' )   
prtFlux ('river in      ', RUN_flx_input      , 'f' )   
prtFlux ('river out     ', RUN_flx_output     , 'f' )   
prtFlux ('river bil     ', RUN_flx_bil        , 'f' )   
prtFlux ('runoff lic    ', ATM_flx_runlic     , 'f' )   

ATM_flx_budget = -ATM_flx_sea + ATM_flx_calving + ATM_flx_runlic #+ ATM_flx_qfonte + RUN_flx_river
echo ('')
echo ('  Global        {:12.3e} kg | {:12.4f} Sv | {:12.4f} m '.format ( ATM_flx_budget , ATM_flx_budget / dtime_sec*1E-9, ATM_flx_budget /ATM_aire_sea_tot/ATM_rho ))

#echo ('  E-P-R         {:12.3e} kg | {:12.4e} Sv | {:12.4f} m '.format ( ATM_flx_emp      , ATM_flx_emp      / dtime_sec*1E-6/ATM_rho, ATM_flx_emp      /ATM_aire_sea_tot/ATM_rho ))

echo (' ')
echo ( '\n====================================================================================' )
echo ( f'--  Atmosphere  -- {Title} ' )
echo ( 'Mass begin = {:12.6e} kg | Mass end = {:12.6e} kg'.format (DYN_mas_wat_beg, DYN_mas_wat_end) )
echo ( 'dmass (atm)  = {:12.3e} kg | {:12.4e} Sv | {:12.4f} mm/year '.format ( dDYN_mas_wat , kg2Sv(dDYN_mas_wat) , kg2myear(dDYN_mas_wat )*1000. ))
echo ( 'Sum wbilo_*  = {:12.3e} kg | {:12.4e} Sv | {:12.4f} mm/year '.format ( ATM_flx_wbilo, kg2Sv(ATM_flx_wbilo), kg2myear(ATM_flx_wbilo )*1000. ))
echo ( 'E-P          = {:12.3e} kg | {:12.4e} Sv | {:12.4f} mm/year '.format ( ATM_flx_emp  , kg2Sv(ATM_flx_emp ) , kg2myear(ATM_flx_emp  )*1000. ))
echo ( ' ' )
prtFlux ( 'Perte eau atm ', ATM_flx_wbilo - dDYN_mas_wat, 'e', True )
echo ( 'Perte eau atm = {:12.3e} (rel)  '.format ( (ATM_flx_wbilo - dDYN_mas_wat)/dDYN_mas_wat  ) )

echo (' ')
prtFlux ( 'Perte eau atm', ATM_flx_emp  - dDYN_mas_wat , 'e', True )
echo ( 'Perte eau atm = {:12.3e} (rel)  '.format ( (ATM_flx_emp-dDYN_mas_wat)/dDYN_mas_wat  ) )
echo (' ')

echo ( '\n====================================================================================' )
echo ( f'-- SECHIBA fluxes  -- {Title} ' )

SRF_flx_rain     = SRF_flux_int ( SRF_rain     )
SRF_flx_evap     = SRF_flux_int ( SRF_evap     )
SRF_flx_snowf    = SRF_flux_int ( SRF_snowf    )
SRF_flx_subli    = SRF_flux_int ( SRF_subli    )
SRF_flx_transpir = SRF_flux_int ( SRF_transpir )
SRF_flx_emp      = SRF_flux_int ( SRF_emp      )

RUN_flx_torouting  =  RUN_flx_runoff  + RUN_flx_drainage
RUN_flx_fromrouting = RUN_flx_coastal + RUN_flx_river

SRF_flx_all = SRF_flx_rain + SRF_flx_snowf - SRF_flx_evap - RUN_flx_runoff - RUN_flx_drainage

prtFlux ('rain       ', SRF_flx_rain     , 'f' )
prtFlux ('evap       ', SRF_flx_evap     , 'f' )
prtFlux ('snowf      ', SRF_flx_snowf    , 'f' )
prtFlux ('E-P        ', SRF_flx_emp      , 'f' )
prtFlux ('subli      ', SRF_flx_subli    , 'f' )
prtFlux ('transpir   ', SRF_flx_transpir , 'f' )
prtFlux ('to routing ', RUN_flx_torouting, 'f' )
prtFlux ('budget     ', SRF_flx_all      , 'f', True )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( 'Water content in surface ' )
echo ( 'SRF_mas_wat_beg  = {:12.6e} kg | SRF_mas_wat_end  = {:12.6e} kg '.format (SRF_mas_wat_beg, SRF_mas_wat_end) )
prtFlux ( 'dMass (water srf)', dSRF_mas_wat, 'e', True)
echo ( 'dMass (water srf) = {:12.4e} (rel)   '.format ( (SRF_flx_all-dSRF_mas_wat)/dSRF_mas_wat) )

echo ('')
echo ( '\n====================================================================================' )
echo ( f'-- RUNOFF fluxes  -- {Title} ' )
RUN_flx_all = RUN_flx_torouting - RUN_flx_river - RUN_flx_coastal
prtFlux ('runoff    ', RUN_flx_runoff      , 'f' ) 
prtFlux ('drainage  ', RUN_flx_drainage    , 'f' ) 
prtFlux ('run+drain ', RUN_flx_torouting   , 'f' ) 
prtFlux ('river     ', RUN_flx_river       , 'f' ) 
prtFlux ('coastal   ', RUN_flx_coastal     , 'f' ) 
prtFlux ('riv+coa   ', RUN_flx_fromrouting , 'f' ) 
prtFlux ('budget    ', RUN_flx_all         , 'f' , True)

echo ( '\n------------------------------------------------------------------------------------' )
echo ( f'Water content in routing  -- {Title} ' )
echo ( 'RUN_mas_wat_beg  = {:12.6e} kg | RUN_mas_wat_end = {:12.6e} kg '.format (RUN_mas_wat_beg, RUN_mas_wat_end) )
prtFlux ( 'dMass (routing)  ', dRUN_mas_wat+dSRF_mas_lake, 'f', True)

echo ( '\n------------------------------------------------------------------------------------' )
echo ( f'Water content in routing  -- {Title} ' )
echo ( 'RUN_mas_wat_beg  = {:12.6e} kg | RUN_mas_wat_end = {:12.6e} kg '.format (RUN_mas_wat_beg, RUN_mas_wat_end) )
prtFlux ( 'dMass (routing)  ', dRUN_mas_wat, 'f', True  )

print ( 'Routing error : {:12.3e} (rel)'.format ( (RUN_flx_all-dRUN_mas_wat)/dRUN_mas_wat ) )

# PRINT *,"routing water Balance ;  before : ", water_balance_before," ; after : ",water_balance_after,  &a
# 1150	            " ; delta : ", 100*(water_balance_after-water_balance_before)/(0.5*(water_balance_after+water_balance_before)),"%"
